

#ifndef OVAL_SYSTEM_CHAR_H_INCLUDED
#define OVAL_SYSTEM_CHAR_H_INCLUDED

#include "ovalcommon.h"

enum OvalObjectFlag_t {
	OVAL_OBJ_ERROR,
	OVAL_OBJ_INCOMPLETE,
	OVAL_OBJ_COMPLETE,
	OVAL_OBJ_DOES_NOT_EXIST,
	OVAL_OBJ_NOT_COLLECTED,
	OVAL_OBJ_NOT_APPLICABLE
};

enum OvalItemStatus_t {
	OVAL_ITEM_ERROR,
	OVAL_ITEM_EXISTS,
	OVAL_ITEM_DOES_NOT_EXIST,
	OVAL_ITEM_NOT_COLLECTED
};

typedef struct SystemCharInterface_s {
	char* name;
	char* ip;
	char* mac;
} SystemCharInterface_t;


typedef struct OvalVariable_s {
	char* id;
	char* value;
} OvalVariable_t;


typedef struct CollectedItem_s {
	
	char* id;
	enum OvalItemStatus_t status;

	OvalMessage_t* messages;
	size_t messages_n;
	size_t messages_alloc_;

} CollectedItem_t;


typedef struct CollectedObject_s {

	char* id;
	enum OvalObjectFlag_t flag;
	int version;

	int variable_instance;
	char* comment;

	OvalMessage_t* messages;
	size_t messages_n;

	OvalVariable_t* variables;
	size_t variables_n;

	CollectedItem_s* references;
	size_t references_n;

	void* priv_; // private implementation data

} CollectedObject_t;


typedef struct SystemChar_s {

	struct {
		char* os_name;
		char* os_version;
		char* architecture;
		char* primary_host_name;
		SysCharInterface_t* interfaces;
		size_t interfaces_n;
		size_t interfaces_alloc;
	} system_info;

	CollectedObject_t** objects;
	size_t objects_n;

	CollectedItem_t** items;
	size_t items_n;

	// Sig_t signature;

	void* priv_; // private implementation data

} SystemChar_t;



SystemChar_t* systemchar_new_file(const char* filename);

SystemChar_t* systemchar_new_empty(void);

SystemChar_t* systemchar_new(const char* os_name, const char* os_version, const char* architecture, const char* primary_host_name);

bool systemchar_add_object(SystemChar_t* syschar, CollectedObject_t* obj);

bool systemchar_add_item(SystemChar_t* syschar, CollectedItem_t* item);

bool systemchar_add_interface(SystemChar_t* syschar, const char* name, const char* ip, const char* mac);

bool systemchar_write(SystemChar_t* syschar, const char* filename);

CollectedObject_t* systemchar_find_object(SystemChar_t* syschar, const char* id);

CollectedItem_t* systemchar_find_item(SystemChar_t* syschar, const char* id);

void systemchar_delete(SystemChar_t* syschar);



CollectedItem_t* collecteditem_new_empty(void);

CollectedItem_t* collecteditem_new(const char* id, enum OvalItemStatus_t status);

void collecteditem_delete(CollectedItem_t* item);



CollectedObject_t* collectedobject_new_empty(void);

CollectedObject_t* collectedobject_new(const char* id, enum OvalObjectFlag_t falg, int version, const char* comment);

OvalMessage_t* collectedobject_add_message(CollectedObject_t* obj, enum OvalMessageLevel_t level, const char* msg);

OvalMessage_t* collectedobject_add_message_ptr(CollectedObject_t* obj, OvalMessage_t* msg);

OvalVariable_t* collectedobject_add_variable(CollectedObject_t* obj, const char* id, const char* value);

const char* collectedobject_find_variable(CollectedObject_t* obj, const char* id);

bool collectedobject_add_reference(CollectedObject_t* obj, CollectedItem_t* item);

void collectedobject_delete(CollectedObject_t* obj);


#endif // OVAL_SYSTEM_CHARACTERISTICS_H_INCLUDED


