/* OVAL Definition document */
#ifndef RH_OVAL_DEFINITIONS
#define RH_OVAL_DEFINITIONS

/// Enumeration of possible definition classes
typedef enum {
	DEFCLS_COMPLIANCE,
	DEFCLS_INVENTORY,
	DEFCLS_MISCELLANEOUS,
	DEFCLS_PATCH,
	DEFCLS_VULNERABILITY
} DefinitionClassEnum_t;

/// Enumeration of affected families
typedef enum {
	AFCFML_CATOS,
	AFCFML_IOS,
	AFCFML_MACOS,
	AFCFML_PIXOS,
	AFCFML_UNDEFINED,
	AFCFML_UNIX,
	AFCFML_WINDOWS
} AffectedFamilyEnum_t;

/// Enumeration of check existence values
typedef enum {
	EXIST_ALL,
	EXIST_ANY,
	EXIST_AT_LEAST_ONE,
	EXIST_NONE,
	EXIST_ONLY_ONE
} ExistenceEnum_t;

/// Enumeration of check values
typedef enum {
	CHECK_ALL,
	CHECK_AT_LEAST_ONE,
	CHECK_NONE_EXIST,
	CHECK_NONE_SATISFY,
	CHECK_ONLY_ONE
} CheckEnum_t;

/// Enumeration of data types
typedef enum {
	DATATYPE_BINARY,
	DATATYPE_BOOLEAN,
	DATATYPE_EVR_STRING,
	DATATYPE_FILESET_REVISION,
	DATATYPE_FLOAT,
	DATATYPE_IOS_VERSION,
	DATATYPE_INT,
	DATATYPE_STRING,
	DATATYPE_VERSION
} DatatypeEnum_t;

/// Enumeration of operators
typedef enum {
	OPERATOR_AND,
	OPERATOR_ONE,
	OPERATOR_OR,
	OPERATOR_XOR
} OperatorEnum_t;

/// Enumeration of operations
typedef enum {
	OPERATION_EQUALS,
	OPERATION_NOT_EQUAL,
	OPERATION_GREATER_THAN,
	OPERATION_LESS_THAN,
	OPERATION_GREATER_THAN_OR_EQUAL,
	OPERATION_LESS_THAN_OR_EQUAL,
	OPERATION_BITWISE_AND,
	OPERATION_BITWISE_OR,
	OPERATION_PATTERN_MATCH,
	OPERATION_CASE_INSENSITIVE_EQUALS,
	OPERATION_CASE_INSENSITIVE_NOT_EQUAL
} OperationEnum_t;

/// Enumeration of arithemtic operations
typedef enum {
	ARIOP_ADD,
	ARIOP_MULTIPLY
} ArithmeticOperationEnum_t;

/// Enumeration of ComponentGroup_t component types
typedef enum {
	CGRP_OBJECT,
	CGRP_VARIABLE,
	CGRP_LITERAL,
	CGRP_FUNCTIONGRP
} ComponentGroupEnum_t;

/// Enumeration of FunctionGroup_t function types
typedef enum {
	FGRP_ARITHMETIC,
	FGRP_BEGIN,
	FGRP_CONCAT,
	FGRP_END,
	FGRP_ESCAPE_REGEX,
	FGRP_SPLIT,
	FGRP_SUBSTRING,
	FGRP_TIME_DIFF,
	FGRP_REGEX_CAPTURE
} FunctionGroupEnum_t;

/// Enumeration of date time formats
typedef enum {
	DTFMT_YMD,
	DTFMT_MDY,
	DTFMT_DMY,
	DTFMT_WIN_FILETIME,
	DTFMT_EPOCH
} DateTimeFormatEnum_t;

/// Enumeration of variable types that can be containd in VariableBaseType_t
typedef enum {
	VARTYPE_EXTERNAL,
	VARTYPE_CONST,
	VARTYPE_LOCAL
} VariableTypeEnum_t;


/**
 * Structure holding data from "generator" element
 */
typedef struct {
	char *product_name;
	char *product_version;
	char *schema_version;
	char *timestamp;
} Generator_t;

/**
 * Structure holding data from "affected" element
 */
typedef struct Affected_s {
	AffectedFamilyEnum_t family;
	char **platform;
	char **product;
	struct Affected_s *next;
} Affected_t;

/**
 * Structure holding data from "reference" element
 */
typedef struct Reference_s {
	char *source;
	char *ref_id;
	char **ref_url;
	struct Reference_s *next;
} Reference_t;

/**
 * Structure holding data from "metadata" element
 */
typedef struct {
	char *title;
	Affected_t *affected;
	Reference_t *reference;
	char *description;
} Metadata_t;

/**
 * Structure holding data from "possible_value" element of an external variable
 */
typedef struct PossibleValue_s {
	char *hint;
	char *value;
	struct PossibleValue_s *next;
} PossibleValue_t;

/**
 * Structure holding data from "restriction" element
 */
typedef struct Restriction_s {
	OperationEnum_t operation;
	char *value;
	struct Restriction_s *next;
} Restriction_t;

/**
 * Structure holding data from "possible_restriction" element of an external variable
 */
typedef struct PossibleRestriction_s {
	char *hint;
	Restriction_t *restriction;
	struct PossibleRestriction_s *next;
} PossibleRestriction_t;

/**
 * Structure holding data from "external_variable" element, part of VariableBaseType_t
 */
typedef struct {
	PossibleValue_t *possible_value;
	PossibleRestriction_t *possible_restriction;
} ExternalVariable_t;

/**
 * Structure holding data from "constan_variable" element, part of VariableBaseType_t
 */
typedef struct {
	char **value;
} ConstantVariable_t;

/**
 * Forward declaration of ComponentGroup_t
 */
typedef struct ComponentGroup_s ComponentGroup_t;

/**
 * Structure holding data of the "object_component" element, part of ComponentGroup_t
 */
typedef struct {
	char *object_ref;
	char *item_field;
} ObjectComponent_t;

/**
 * Structure holding data of the "variable_component" element, part of ComponentGroup_t
 */
typedef struct {
	char *var_ref;
} VariableComponent_t;

/**
 * Structure holding data of the "literal_component" element, part of ComponentGroup_t
 */
typedef struct {
	char *value;
	DatatypeEnum_t datatype;
} LiteralComponent_t;

/**
 * Structure holding data of the "arithmetic" element, part of FunctionGroup_t
 */
typedef struct {
	ArithmeticOperationEnum_t arithmetic_operation;
	ComponentGroup_t *component_group;
} ArithmeticFunction_t;

/**
 * Structure holding data of the "begin" element, part of FunctionGroup_t
 */
typedef struct {
	char *character;
	ComponentGroup_t *component_group;
} BeginFunction_t;

/**
 * Structure holding data of the "concat" element, part of FunctionGroup_t
 */
typedef struct {
	ComponentGroup_t *component_group;
} ConcatFunction_t;

/**
 * Structure holding data of the "end" element, part of FunctionGroup_t
 */
typedef struct {
	char *character;
	ComponentGroup_t *component_group;
} EndFunction_t;

/**
 * Structure holding data of the "escape_regex" element, part of FunctionGroup_t
 */
typedef struct {
	ComponentGroup_t *component_group;
} EscapeRegexFunction_t;

/**
 * Structure holding data of the "split" element, part of FunctionGroup_t
 */
typedef struct {
	char *delimiter;
	ComponentGroup_t *component_group;
} SplitFunction_t;

/**
 * Structure holding data of the "substring" element, part of FunctionGroup_t
 */
typedef struct {
	int substring_start;
	int substring_end;
	ComponentGroup_t *component_group;
} SubstringFunction_t;

/**
 * Structure holding data of the "time_difference" element, part of FunctionGroup_t
 */
typedef struct {
	DateTimeFormatEnum_t format_1;
	DateTimeFormatEnum_t format_2;
	ComponentGroup_t *component_group;
} TimeDiffFunction_t;

/**
 * Structure holding data of the "regex_capture" element, part of FunctionGroup_t
 */
typedef struct {
	char *pattern;
	ComponentGroup_t *component_group;
} RegexCaptureFunction_t;

/**
 * Structure holding data of the "FunctionGroup" group, part of ComponentGroup_t
 */
typedef struct {
	FunctionGroupEnum_t function_type;
	union {
		ArithmeticFunction_t *arithmetic_function;
		BeginFunction_t *begin_function;
		ConcatFunction_t *concat_function;
		EndFunction_t *end_function;
		EscapeRegexFunction_t *escape_regex_function;
		SplitFunction_t *split_function;
		SubstringFunction_t *substring_function;
		TimeDiffFunction_t *time_diff_function;
		RegexCaptureFunction_t *regex_capture_function;
	} function;
} FunctionGroup_t;

/**
 * Structure holding data of the "ComponentGroup" group
 */
struct ComponentGroup_s {
	ComponentGroupEnum_t component_type;
	union {
		ObjectComponent_t *object_component;
		VariableComponent_t *variable_component;
		LiteralComponent_t *literal_component;
		FunctionGroup_t *function_group;
	} component;
	struct ComponentGroup_s *next;
};

/**
 * Structure holding data of the "local_variable" element, part of VariableBaseType_t
 */
typedef struct {
	ComponentGroup_t *component_group;
} LocalVariable_t;

/**
 * Structure holding common data for all variable types
 */
typedef struct VariableBaseType_s {
	char *id;
	unsigned int varsion;
	DatatypeEnum_t datatype;
	char *comment;
	int deprecated;
	VariableTypeEnum_t variable_type;
	union {
		ExternalVariable_t *external_var;
		ConstantVariable_t *const_var;
		LocalVariable_t *local_var;
	} variable;
	// todo: signature
	struct VariableBaseType_s *next;
} VariableBaseType_t;


/**
 * Structure holding data of the "object" element
 */
typedef struct Object_s{
	char *id;
	unsigned int version;
	char *comment;
	int deprecated;
	char **notes;
	// todo: signature
	struct Object_s *next;
} Object_t;

/**
 * Structure holding data of the "state" element
 */
typedef struct State_s {
	char *id;
	unsigned int version;
	OperatorEnum_t operator;
	char *comment;
	int deprecated;
	// todo: signature
	char **notes;
} State_t;

/**
 * Structure holding data of the "test" element
 */
typedef struct Test_s {
	char *id;
	unsigned int version;
	ExistenceEnum_t check_existence;
	CheckEnum_t check;
	char *comment;
	int deprecated;
	char **notes;
	// todo: signature
	struct Test_s *next;
} Test_t;

/**
 * Structure holding data of the "criterion" element
 */
typedef struct Criterion_s {
	char *test_ref;
	int negate;
	char *comment;
	struct Criterion_s *next;
} Criterion_t;

/**
 * Structure holding data of the "extend_definition" element
 */
typedef struct ExtendDefinition_s {
	char *definition_ref;
	int negate;
	char *comment;
	struct ExtendDefinition_s *next;
} ExtendDefinition_t;

/**
 * Structure holding data of the "criteria" element
 */
typedef struct Criteria_s {
	OperatorEnum_t operator;
	int negate;
	char *comment;
	struct Criteria_s *criteria;
	Criterion_t *criterion;
	ExtendDefinition_t *extend_definition;
	struct Criteria_s *next;
} Criteria_t;

/**
 * Structure holding data of the "definition" element
 */
typedef struct Definition_s {
	char *id;
	unsigned int version;
	DefinitionClassEnum_t class;
	int deprecated;
	Metadata_t *metadata;
	char **notes;
	Criteria_t *criteria;
	// todo: signature
	struct Definition_s *next;
} Definition_t;
#endif

